# Copyright 2022 Tellius, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
from pathlib import Path

from tellius_data_manager.persistence_operators.dataframe_operators.dataframe_readers.dataframe_reader_factory import \
    DataframeReaderFactory
from tellius_data_manager.persistence_operators.dataframe_operators.dataframe_readers.s3_csv_reader import (
    S3CSVReader,
)
from tellius_data_manager.pipes.readers.reader_pipe import ReaderPipe
from tellius_data_manager.utils import dict2obj


class ReadS3(ReaderPipe):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._reader = DataframeReaderFactory.generate(configuration=kwargs["reader"])

    def _run(self, **kwargs) -> ReaderPipe:
        filename = kwargs.get("filename")

        df = self._reader.execute(
            **{
                "filename": filename,
            }
        )

        self._state.update_metadata(key="data", value=df)

        return self
